@@ -4,9 +4,10 @@ import anyio
 import click
 import functools
 import inspect
+import typing
 import typing as t
 
-if t.TYPE_CHECKING:
+if typing.TYPE_CHECKING:
     from kupala.application import Kupala
 
 
@@ -14,7 +15,7 @@ def create_dispatcher(app: Kupala, fn: t.Callable) -> t.Callable:
     fn_kwargs = t.get_type_hints(fn)
 
     @functools.wraps(fn)
-    def dispatcher(*args: t.Any, **kwargs: t.Any) -> t.Any:
+    def dispatcher(**kwargs: t.Any) -> t.Any:
         for name, class_name in fn_kwargs.items():
             if hasattr(class_name, 'from_app'):
                 kwargs[name] = class_name.from_app(app)
@@ -22,11 +23,11 @@ def create_dispatcher(app: Kupala, fn: t.Callable) -> t.Callable:
         if inspect.iscoroutinefunction(fn):
 
             async def async_dispatcher() -> t.Any:
-                return await app.invoke(fn, extra_kwargs=kwargs)
+                return await fn(**kwargs)
 
             return anyio.run(async_dispatcher)
         else:
-            return app.invoke(fn, extra_kwargs=kwargs)
+            return fn(**kwargs)
 
     return dispatcher
 
