@@ -10,11 +10,12 @@ import uuid
 from babel.core import Locale
 from imia import UserLike, UserToken
 from starlette import datastructures as ds, requests
+from starlette.datastructures import State
 from starlette.requests import empty_receive, empty_send
 from starlette.types import Receive, Scope, Send
 from starsessions import Session
 
-from kupala.storages.storages import Storage, Storages
+from kupala.storages.storages import Storage
 
 if t.TYPE_CHECKING:
     from .application import Kupala
@@ -132,9 +133,9 @@ class UploadFile(ds.UploadFile):
 
         storage: Storage
         if disk:
-            storage = self.request.app.resolve(Storages).get(disk)
+            storage = self.request.app.storages.get(disk)
         else:
-            storage = self.request.app.resolve(Storage)
+            storage = self.request.app.storages.get_default()
         await storage.put(file_path, await self.read())
         return file_path
 
@@ -164,7 +165,11 @@ class Headers(requests.Headers):
     pass
 
 
-class Request(requests.Request):
+_A = t.TypeVar('_A', bound='Kupala')
+_S = t.TypeVar('_S', bound=State)
+
+
+class Request(requests.Request, t.Generic[_A, _S]):
     def __new__(cls, scope: Scope, receive: Receive = empty_receive, send: Send = empty_send) -> Request:
         if 'request' not in scope:
             instance = super().__new__(cls)
@@ -182,7 +187,7 @@ class Request(requests.Request):
         return QueryParams(super().query_params)
 
     @property
-    def app(self) -> 'Kupala':
+    def app(self) -> _A:
         return self.scope['app']
 
     @property
@@ -262,6 +267,10 @@ class Request(requests.Request):
             self.scope['_form_old_input'] = old_form_input
         return self.scope['_form_old_input']
 
+    @property
+    def state(self) -> _S:
+        return t.cast(_S, super().state)
+
     async def remember_form_data(self) -> None:
         """Flush current form data into session so it can be used on the next page to render form errors."""
         if 'session' in self.scope:
